@@ -1,4 +1,4 @@
-from typing import Callable, Union, Any, Dict
+from typing import Callable, Union, Any, Dict, Type, List
 from dataclasses import dataclass, field
 from os import environ as env
 
@@ -214,3 +214,90 @@ class TokenValidator:
     def __post_init__(self):
         if self.auto_init:
             self.approve_or_deny()
+
+
+@dataclass(init=True)
+class StrategyResult:
+
+    is_valid: bool = False
+    message: str = "Not Implemented!"
+    token: str = ""
+
+
+@dataclass(init=True)
+class StrategyError:
+
+    message: str
+    exception: Exception
+
+    kwargs: dict = None
+    args: list = None
+
+
+class Strategy:
+
+    fail_safe: bool = True
+    errors: list = []
+
+    def validate(self):
+
+        rules = [r for r in dir(self) if r.startswith("must_")]
+
+        for rule in rules:
+            rule_fn = getattr(self, rule)
+            try:
+
+                if callable(rule_fn):
+                    rule_fn()
+
+            except Exception as e:
+                self.errors.append(StrategyError(message=f"{e}",
+                                                 exception=e))
+                if not self.fail_safe:
+                    break
+
+        if self.errors:
+            raise ValueError("\n".join([e.message for e in self.errors]))
+
+        return True
+
+
+@dataclass(init=True)
+class StepError:
+    errors: list
+    name: str
+
+
+class MasterPlan:
+
+    strategy_steps: List[Type[Strategy]] = None
+    fail_safe: bool = True
+    errors: list = []
+
+    def add_error(self, errors, name):
+        self.errors.append(StepError(errors=errors, name=name))
+
+    def run(self):
+        for step in self.strategy_steps:
+            if issubclass(step, Strategy):
+                strategy = step()
+                try:
+                    strategy.validate()
+                except Exception as e:
+
+                    self.add_error(strategy.errors, step.__name__)
+
+                    if not self.fail_safe:
+                        raise e
+
+
+class CognitoJWKStrategy(Strategy):
+    ...
+
+
+class CognitoIssuerStrategy(Strategy):
+    ...
+
+
+class Issuer:
+    ...
